@@ -5,7 +5,7 @@ use warnings;
 package Marlin::XAttribute::Alias;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.011002';
+our $VERSION   = '0.012000';
 
 use Eval::TypeTiny ();
 use Role::Tiny;
@@ -47,59 +47,6 @@ after install_accessors => sub {
 	$me->install_coderef( $_, $coderef ) for @aliases;
 };
 
-before add_code_for_initialization => sub {
-	my ( $me, $code ) = @_;
-	
-	my $init_arg = exists($me->{init_arg}) ? $me->{init_arg} : $me->{slot};
-	return unless defined $init_arg;
-	
-	my @aliases = @{ $me->{':Alias'}{alias} };
-	
-	# If the caller provided the init_arg as expected, then need to
-	# check that no aliases were used!
-	$code->addf( 'if ( exists $args{%s} ) {', B::perlstring($init_arg) );
-	$code->increase_indent;
-	my $check = do {
-		my $enum = Types::Common::Enum->of( @aliases );
-		$enum->can( '_regexp' )
-			? sprintf( '/\\A%s\\z/', $enum->_regexp )
-			: $enum->inline_check( '$_' );
-	};
-	$code->addf( 'my @superfluous = grep { %s } keys %%args;', $check );
-	$code->addf( '%s("Superfluous %%s used for attribute \'%%s\': %%s" , @superfluous==1 ? "alias" : "aliases", %s, join( q[, ], sort @superfluous ) ) if @superfluous;', $me->_croaker, B::perlstring($me->{slot}) );
-	$code->decrease_indent;
-	$code->addf( '}' );
-	$code->addf( 'else {' );
-	$code->increase_indent;
-	$code->addf( 'my $found;' );
-	$code->addf( 'ALIAS: for my $alias ( %s ) {', join q{, } => map B::perlstring($_), @aliases );
-	$code->increase_indent;
-	$code->addf( 'if ( exists $args{$alias} ) {' );
-	$code->increase_indent;
-	$code->addf( '%s("Superfluous alias used for attribute \'%%s\': %%s", %s, $alias ) if defined $found;', $me->_croaker, B::perlstring($me->{slot}) );
-	$code->addf( '$args{%s} = delete $args{$alias};', B::perlstring($init_arg) );
-	$code->addf( '$found = $args{%s} = $alias;', B::perlstring( ':used_alias_for_' . $init_arg ) );
-	$code->decrease_indent;
-	$code->addf( '}' );
-	$code->decrease_indent;
-	$code->addf( '}' );
-	$code->decrease_indent;
-	$code->addf( '}' );
-};
-
-after add_code_for_initialization => sub {
-	my ( $me, $code ) = @_;
-	
-	my $init_arg = exists($me->{init_arg}) ? $me->{init_arg} : $me->{slot};
-	return unless defined $init_arg;
-	
-	$code->addf( 'if ( exists $args{%s} ) {', B::perlstring( ':used_alias_for_' . $init_arg ) );
-	$code->increase_indent;
-	$code->addf( '$args{delete $args{%s}} = delete $args{%s};', B::perlstring( ':used_alias_for_' . $init_arg ), B::perlstring($init_arg) );
-	$code->decrease_indent;
-	$code->addf( '}' );
-};
-
 around allowed_constructor_parameters => sub {
 	my $next = shift;
 	my $me = shift;
@@ -227,7 +174,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2025 by Toby Inkster.
+This software is copyright (c) 2025-2026 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
